-- ============================================================================
-- == HalonaTools - A collection of functions used to help users monitor     ==
-- ==               their buffs.                                             ==
-- ============================================================================

HTBuffMonitor = LibStub("AceAddon-3.0"):NewAddon("HTBuffMonitor", "AceConsole-3.0", "AceEvent-3.0")
local L       = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBuffMonitor:OnInitialize() 
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBuffMonitor:OnEnable()

  self:RegisterEvent("READY_CHECK")

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBuffMonitor:OnDisable()
end

-- ============================================================================
-- == DoMissingBuffCheck(characterName, channelName)                         ==
-- ==                                                                        ==
-- == This function will check the named character for missing buffs.        ==
-- ============================================================================

function HTBuffMonitor:DoMissingBuffCheck(characterName, channelName)

  -- ConvergeFu.db.profile.bmIssuesFound = L["common-data-option-no"]

  local hasSuitableFlask          = HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)
  local hasSuitableBattleElixir   = HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)
  local hasSuitableGuardianElixir = HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)
  local hasSuitableFoodBuff       = HTBuffMonitor:IsBuffedWithSuitableFood(characterName)
  local hasSuitablePoisonBuffs    = HTBuffMonitor:IsBuffedWithSuitableRoguePoisons(characterName)
   
  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a flask or two elixirs activated.          --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFlask then

    -- == ---------------------------------------------------------------------
    -- == -- Has guardian elixir but suitable battle elixir missing.         --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and     hasSuitableGuardianElixir then
      if HTConfig:IsEnabledBuffMonitorCheckFlasks(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-battle-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- Has battle elixir but suitable guardian elixir missing.         --
    -- == ---------------------------------------------------------------------
    if      hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:IsEnabledBuffMonitorCheckBattleElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-guardian-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end 

    -- == ---------------------------------------------------------------------
    -- == -- No flask, no battle elixir, no guardian elixir.                 --
    -- == ---------------------------------------------------------------------            
    if  not hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:IsEnabledBuffMonitorCheckGuardianElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-no-flask-or-elixirs"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end            
   
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a food-buff activated.                     --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFoodBuff then
    if HTConfig:IsEnabledBuffMonitorCheckFood(nil) then
      HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-food"], "CHARACTER_NAME", characterName), channelName, nil, nil)
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has suitable rogue poisons activated.          --
  -- == -----------------------------------------------------------------------
  if UnitClass(characterName) == L["all-data-classname-rogue"] then
    if not hasSuitablePoisonBuffs then
      if HTConfig:IsEnabledBuffMonitorCheckPoisons(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-weapon-poisons"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end
  end
   
  -- == -----------------------------------------------------------------------
  -- == -- Check if character buffs are nearing expiry.                      --
  -- == -----------------------------------------------------------------------
  if HTConfig:GetIsEnabledBuffMonitorExpiryTimeCheck(nil) then
     HTBuffMonitor:CheckExpiryTimes(characterName, channelName)
  end

end

-- ============================================================================
-- == IsCharacterBuffedWithSuitableFlask(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable flask activated.                                              ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFlasksArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableBattleElixir(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable battle elixir activated.                                      ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableBattleElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableGuardianElixir(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable guardian elixir activated.                                    ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableGuardianElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableFood(characterName)                                ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable food buff activated.                                          ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFood(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFoodArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableRoguePoisons(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable food buff activated.                                          ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableRoguePoisons(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableRoguePoisonsArray())

end

-- ============================================================================
-- == IsBuffedWith(characterName, suitableBuffs)                             ==
-- ==                                                                        ==
-- == This function performs a check on the character identified by the      ==
-- == supplied parameter to determine if that character is currently buffed  ==
-- == with any of the buffs detailed in the supplied 'suitableBuffs' table.  ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWith(characterName, suitableBuffs)
   
  -- == ----------------------------------------------------------------------
  -- == -- Perform the test.                                                --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName = UnitBuff(characterName, b)
    if buffName then
      for i,v in ipairs(suitableArray) do
        if suitableBuffs[i] == buffName then
          return true
        end
      end
    end
  end
   
  return false
   
end

-- ============================================================================
-- == CheckExpiryTimes(characterName, channelName)                           ==
-- ==                                                                        ==
-- == This function simply checks the expiry times of the buff-types that    ==
-- == HalonaTools monitors and reports any buffs that will expire within a   ==
-- == prescribed time-limit to the nominated channel.                        ==
-- ============================================================================

function HTBuffMonitor:CheckExpiryTimes(characterName, channelName)

  HTConsole:Debug("HTBuffMonitor:CheckExpiryTimes("..characterName..", "..channelName..")")
   
  -- == ----------------------------------------------------------------------
  -- == -- Setup the 'suitable' buff array and other required properties.   --
  -- == ----------------------------------------------------------------------
  local suitableFlasksArray          = HTBuffMonitor:GetSuitableFlasksArray()
  local suitableBattleElixirsArray   = HTBuffMonitor:GetSuitableBattleElixirsArray()
  local suitableGuardianElixirsArray = HTBuffMonitor:GetSuitableGuardianElixirsArray()
  local suitableFoodArray            = HTBuffMonitor:GetSuitableFoodArray()
  local suitableRoguePoisonsArray    = HTBuffMonitor:GetSuitableRoguePoisonsArray()
   
  local timeLimitMinutes             = HTConfig:GetBuffMonitorExpiryTimeLimit(nil)
   
  local suitableBuffsArray           = {}
   
  if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFlasksArray)
  end
   
  if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableBattleElixirsArray)
  end
  
  if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableGuardianElixirsArray)
  end
   
  if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFoodArray)
  end
   
  if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableRoguePoisonsArray)
  end
   
  -- == ----------------------------------------------------------------------
  -- == -- Perform the tests.                                               --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName, buffRank, buffIcon, buffCount, buffDispelType, buffDuration, buffExpires, buffCaster, buffIsStealable = UnitBuff(characterName, b)
    if buffName then
      if buffDuration then
        for i,v in ipairs(suitableBuffsArray) do
          if suitableBuffsArray[i] == buffName then
                 
            remainingTime = HTData:SafeNumber(tonumber(format("%.2f",-1*(GetTime()-buffExpires)/60)))
                  
            HTConsole:Debug("Buff: "..buffName)
            HTConsole:Debug("Remaining Time: "..remainingTime..", timeLimitMinutes: "..HTData:SafeNumber(timeLimitMinutes))

            if remainingTime < timeLimitMinutes then
              local messageText = string.gsub(L["bfm-short-buff-duration-announce"], "CHARACTER_NAME", characterName)
              messageText       = string.gsub(messageText, "BUFF_NAME", buffName)
              messageText       = string.gsub(messageText, "TIME_LIMIT", timeLimitMinutes)
            
              HTConsole:SendChatMessage(messageText, channelName, nil, nil)
              
              -- ConvergeFu:SendBMNotificationByAddon(characterName, L["tools-bm-your"].." "..buffName..L["tools-bm-buff-expiry-msg-01"]..timeLimitMinutes..L["tools-bm-buff-expiry-msg-02"])
              -- ConvergeFu.db.profile.bmIssuesFound = L["common-data-option-yes"]
            end
          end
        end
      end
    end
  end
   
end

-- ============================================================================
-- == GetSuitableFlasksArray()                                               ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == flasks with respect to PVE raid usage.                                 ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFlasksArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}
  
  -- Add data for the Cataclysm expansion.
  if HTConfig:IsEnabledBuffMonitorCataFlasks(nil) then
    table.insert(suitableBuffs, GetSpellInfo(53760))  -- Flask of Endless Rage.
    table.insert(suitableBuffs, GetSpellInfo(53755))  -- Flask of the Frost Wyrm.
    table.insert(suitableBuffs, GetSpellInfo(54212))  -- Flask of Pure Mojo.
    table.insert(suitableBuffs, GetSpellInfo(53758))  -- Flask of Stoneblood.
    table.insert(suitableBuffs, GetSpellInfo(62380))  -- Lesser Flask of Resistance.
  end
  
  -- Add data for the Wrath expansion.
  if HTConfig:IsEnabledBuffMonitorWrathFlasks(nil) then
    table.insert(suitableBuffs, GetSpellInfo(53760))  -- Flask of Endless Rage.
    table.insert(suitableBuffs, GetSpellInfo(53755))  -- Flask of the Frost Wyrm.
    table.insert(suitableBuffs, GetSpellInfo(54212))  -- Flask of Pure Mojo.
    table.insert(suitableBuffs, GetSpellInfo(53758))  -- Flask of Stoneblood.
    table.insert(suitableBuffs, GetSpellInfo(62380))  -- Lesser Flask of Resistance.
  end  
   
  return suitableBuffs
   
end

-- ============================================================================
-- == GetSuitableBattleElixirsArray()                                        ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == battle elixirs with respect to PVE raid usage.                         ==
-- ============================================================================

function HTBuffMonitor:GetSuitableBattleElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}
  
  -- Add data for the Cataclysm expansion.
  if HTConfig:IsEnabledBuffMonitorCataBattleElixirs() then
  
  end
  
  -- Add data for the Wrath expansion.
  if HTConfig:IsEnabledBuffMonitorWrathBattleElixirs() then   
    table.insert(suitableBuffs, GetSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, GetSpellInfo(60345)) -- Armor Piercing.
    table.insert(suitableBuffs, GetSpellInfo(60341)) -- Deadly Strikes.
    table.insert(suitableBuffs, GetSpellInfo(60344)) -- Expertise.
    table.insert(suitableBuffs, GetSpellInfo(53749)) -- Guru's Elixir.
    table.insert(suitableBuffs, GetSpellInfo(60346)) -- Lightning Speed.
    table.insert(suitableBuffs, GetSpellInfo(28497)) -- Mighty Agility.
    table.insert(suitableBuffs, GetSpellInfo(53748)) -- Mighty Strength.
    table.insert(suitableBuffs, GetSpellInfo(33721)) -- Spellpower Elixir.
    table.insert(suitableBuffs, GetSpellInfo(53746)) -- Wrath Elixir.
  end
   
  return suitableBuffs
   
end

-- ============================================================================
-- == GetSuitableGuardianElixirsArray()                                      ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == guardian elixirs with respect to PVE raid usage.                       ==
-- ============================================================================

function HTBuffMonitor:GetSuitableGuardianElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}
  
  -- Add data for the Cataclysm expansion.
  if HTConfig:IsEnabledBuffMonitorCataGuardianElixirs() then
  
  end
  
  -- Add data for the Wrath expansion.
  if HTConfig:IsEnabledBuffMonitorWrathGuardianElixirs() then   
    table.insert(suitableBuffs, GetSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, GetSpellInfo(53763)) -- Wrath - Elixir of Protection.
    table.insert(suitableBuffs, GetSpellInfo(53747)) -- Wrath - Elixir of Spirit.
    table.insert(suitableBuffs, GetSpellInfo(60343)) -- Wrath - Mighty Defense.
    table.insert(suitableBuffs, GetSpellInfo(53751)) -- Wrath - Mighty Fortitude.
    table.insert(suitableBuffs, GetSpellInfo(53764)) -- Wrath - Mighty Mageblood.
    table.insert(suitableBuffs, GetSpellInfo(60347)) -- Wrath - Mighty Thoughts.
  end
   
  return suitableBuffs
   
end

-- ============================================================================
-- == GetSuitableFoodArray()                                                 ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == food related buffs with respect to PVE raid usage.                     ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFoodArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}
   
  -- Add data for the Cataclysm expansion.
  if HTConfig:IsEnabledBuffMonitorCataFood() then
  
  end
  
  -- Add data for the Wrath expansion.
  if HTConfig:IsEnabledBuffMonitorWrathFood() then   
    table.insert(suitableBuffs, GetSpellInfo(35272)) -- Well Fed
  end
   
  return suitableBuffs
   
end

-- ============================================================================
-- == GetSuitableRoguePoisonsArray()                                         ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == rogue poisons for end-game content.                                    ==
-- ============================================================================

function HTBuffMonitor:GetSuitableRoguePoisonsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:IsEnabledBuffMonitorCataPoisons() then
  
  end
  
  -- Add data for the Wrath expansion.
  if HTConfig:IsEnabledBuffMonitorWrathPoisons() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(3408))  -- Classic -  Crippling Poison
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(5760))  -- Classic -  Mind Numbing Poison
      
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57981)) -- Wrath   -  Anaesthetic Poison II
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57970)) -- Wrath   -  Deadly Poison IX
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57965)) -- Wrath   -  Instant Poison IX
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57975)) -- Wrath   -  Wound Poison VII
  end
   
  return suitableBuffs
   
end

-- ============================================================================
-- == GetSpellNameFromSpellInfo(spellId)                                     ==
-- ==                                                                        ==
-- == This function simply returns the name of a spell when supplied with    ==
-- == a spellId by using the standard API function GetSpellInfo(spellId) and ==
-- == retrieving the name from the data returned.                            ==
-- ============================================================================
function HTBuffMonitor:GetSpellNameFromSpellInfo(spellId)

   spellName, spellRank, spellIcon, spellPowerCost, spellIsFunnel, spellPowerType, spellCastingTime, spellMinRange, spellMaxRange = GetSpellInfo(spellId) 
   
   spellName = HTData:SafeString(spellName)
   
   return spellName
   
end

-- ============================================================================
-- == READY_CHECK(sender)                                                    ==
-- ==                                                                        ==
-- == This function is triggered when a ready check is fired.                ==
-- ============================================================================
function HTBuffMonitor:READY_CHECK(sender)

   HTBuffMonitor:DoSelfCheck()
   
end

-- ============================================================================
-- == DoSelfCheck()                                                          ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window.                                                       ==
-- ============================================================================

function HTBuffMonitor:DoSelfCheck()
 
   local characterName = UnitName("player")
   
   HTConsole:SendChatMessage(L["bfm-readycheck-msg-01"], "CHAT", nil, nil)
   HTBuffMonitor:DoMissingBuffCheck(characterName, "CHAT")
   HTConsole:SendChatMessage(L["bfm-readycheck-msg-02"], "CHAT", nil, nil)
   
end