-- ============================================================================
-- == HTData - A collection of helpful data manipulation functions.          ==
-- ============================================================================

HTData = LibStub("AceAddon-3.0"):NewAddon("HTData", "AceConsole-3.0", "AceEvent-3.0")
local L   = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTData:OnInitialize() 
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTData:OnEnable()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTData:OnDisable()
end

-- ============================================================================
-- ==  PadLeadingZeros(number, length)                                       ==
-- ==                                                                        ==
-- == This function pads a number with leading zeros to ensure it represents ==
-- == a string with length matching the supplied length parameter.           ==
-- ==                                                                        ==
-- == TODO: This currently ignores the length parameter and assumes the      ==
-- ==       length value should be '2'. Update to use length parameter.      ==
-- ============================================================================

function HTData:PadLeadingZeros(number, length)

  if number < 10 then
    number = "0"..number
  end
  
  return number

end

-- ============================================================================
-- == HTData:StringSplit(text, separator)                                    ==
-- ==                                                                        ==
-- == This function receives a string and converts it into a list of         ==
-- == arguments, using a supplied separator character.                       ==
-- ============================================================================

function HTData:StringSplit(text, separator)
 
   if text == nil then
     text = ""
   end
   
   local args = {}
   local i = 0

   -- search for separators in the string and return the separated data
   for value in string.gmatch(text, "[^"..separator.."]+") do
      i = i + 1
      args[i] = value
   end

   return args
   
end

-- ============================================================================
-- ==  AsResourceName(text)                                                  ==
-- ==                                                                        ==
-- == This function converts the supplied text into a format suitable for    ==
-- == use as all or part of a resource name - that is, the key used to       ==
-- == reference values in the localisation files.                            ==
-- ============================================================================

function HTData:AsResourceName(text)

  text = string.lower(text)
  text = string.gsub(text, " ", "")
  
  return text
  
end

-- ============================================================================
-- == SafeString(value)                                                      ==
-- ==                                                                        ==
-- == Checks supplied parameter, replaces unsafe values as required.         ==
-- ============================================================================

function HTData:SafeString(value)

  if value == nil then
    value = ""
  end
  
  return value
  
end

-- ============================================================================
-- == SafeNumber(value)                                                      ==
-- ==                                                                        ==
-- == Checks supplied parameter, replaces unsafe values as required.         ==
-- ============================================================================

function HTData:SafeNumber(value)

  if value == nil then
    value = "0"
  end
  
  return value
  
end

-- ============================================================================
-- == SafeNumericBoolean(value)                                              ==
-- ==                                                                        ==
-- == Checks supplied parameter, replaces unsafe values as required.         ==
-- ============================================================================

function HTData:SafeNumericBoolean(value)

  if value == nil then
    value = "0"
  else
    value = "1"
  end
  
  return value
  
end

-- ============================================================================
-- == BooleanAsText(value)                                                   ==
-- ==                                                                        ==
-- == Expresses a boolean as text. True maps to "Yes", false to "No"         ==
-- ============================================================================

function HTData:BooleanAsText(value)

  if value == nil then
    value = L["data-conversion-boolean-as-text-yes"]
  end
  
  if value == true then
    value = L["data-conversion-boolean-as-text-yes"]
  else
    value = L["data-conversion-boolean-as-text-no"]
  end
  
end

-- ============================================================================
-- == JoinTables(table1, table2)                                             ==
-- ==                                                                        ==
-- == This function simply joins two tables together and returns the result. ==
-- ============================================================================

function HTData:JoinTables(table1, table2)
   
   for k,v in ipairs(table2) do
      table.insert(table1, v)
   end
   
   return table1   
   
end